export const Anchor = {
	TopLeft: 'TopLeft',
	TopCenter: 'TopCenter',
	TopRight: 'TopRight',
	BottomLeft: 'BottomLeft',
	BottomCenter: 'BottomCenter',
	BottomRight: 'BottomRight'
} as const;

export type AnchorType = typeof Anchor;
export type AnchorName = keyof AnchorType;
export type AnchorPoint = AnchorType[AnchorName];

function offsetStyle(pixels: number, varType: string, sign: number): string {
	const signValue = sign > 0 ? '+' : '-';
	return `calc(${pixels}px ${signValue} var(--component-gap-${varType}))`;
}

export function calculatePosition(
	anchor: AnchorPoint,
	trigger: HTMLDivElement,
	dialog: HTMLDivElement
) {
	const triggerRect = trigger.getBoundingClientRect();
	const dialogRect = dialog.getBoundingClientRect();

	let left: number;
	let top: number;
	let vertSign = 1;
	let horizSign = 1;

	switch (anchor) {
		case Anchor.TopLeft:
			left = triggerRect.left;
			top = triggerRect.top - dialogRect.height;
			horizSign = 1;
			vertSign = -1;
			break;
		case Anchor.TopCenter:
			left = triggerRect.left + triggerRect.width / 2 - dialogRect.width / 2;
			top = triggerRect.top - dialogRect.height;
			horizSign = 0;
			vertSign = -1;
			break;
		case Anchor.TopRight:
			left = triggerRect.right - dialogRect.width;
			top = triggerRect.top - dialogRect.height;
			horizSign = -1;
			vertSign = -1;

			break;
		case Anchor.BottomLeft:
			left = triggerRect.left;
			top = triggerRect.bottom;
			horizSign = 1;
			vertSign = 1;

			break;
		case Anchor.BottomCenter:
			left = triggerRect.left + triggerRect.width / 2 - dialogRect.width / 2;
			top = triggerRect.bottom;
			horizSign = 0;
			vertSign = 1;
			break;
		case Anchor.BottomRight:
			left = triggerRect.right - dialogRect.width;
			top = triggerRect.bottom;
			horizSign = -1;
			vertSign = 1;
			break;
	}

	// Adjust for window boundaries
	if (left < 0) left = 0;
	if (left + dialogRect.width > window.innerWidth) {
		left = window.innerWidth - dialogRect.width;
	}
	if (top < 0) top = 0;
	if (top + dialogRect.height > window.innerHeight) {
		top = window.innerHeight - dialogRect.height;
	}

	if (vertSign) {
		dialog.style.top = offsetStyle(top, 'vertical', vertSign);
	} else {
		dialog.style.top = `${top}px`;
	}

	if (horizSign) {
		dialog.style.left = offsetStyle(left, 'horizontal', horizSign);
	} else {
		dialog.style.left = `${left}px`;
	}
}
